/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright (c) 2011 Oracle and/or its affiliates. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License.  You can
 * obtain a copy of the License at
 * https://glassfish.dev.java.net/public/CDDL+GPL_1_1.html
 * or packager/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 *
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at packager/legal/LICENSE.txt.
 *
 * GPL Classpath Exception:
 * Oracle designates this particular file as subject to the "Classpath"
 * exception as provided by Oracle in the GPL Version 2 section of the License
 * file that accompanied this code.
 *
 * Modifications:
 * If applicable, add the following below the License Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyright [year] [name of copyright owner]"
 *
 * Contributor(s):
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */


import org.glassfish.embeddable.*;

import java.io.File;
import java.lang.reflect.Method;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URL;
import java.net.URLClassLoader;

/**
 * @author Sanjeeb.Sahoo@Sun.COM
 */
public class Issue16501 {
    public static void main(String[] args) throws Exception {
        File installRoot = new File(args[0]);
        File instanceRoot = new File(args[1]);
        URI appURI = URI.create(args[2]);
        for (int i = 0 ; i < 2; ++i) {
            test(installRoot, instanceRoot, appURI, i);
        }
    }

    private static void test(File installRoot, File instanceRoot, URI appURI, int i) throws Exception {
        // Use reflection to load and use Launcher class.
        ClassLoader cl = createCL(installRoot);
        final Class<?> launcherClass = cl.loadClass(Launcher.class.getName());
        Object launcher = launcherClass.newInstance();
        Method method = launcherClass.getMethod("launch", File.class, File.class, URI.class);
        System.out.println("***********************Iteration " + i + "******************");
        method.invoke(launcher, installRoot, instanceRoot, appURI);
        Thread.sleep(1000);
    }

    public static class Launcher {
        public void launch(File installRoot, File instanceRoot, URI appURI) throws GlassFishException {
            BootstrapProperties bs = new BootstrapProperties();
            bs.setInstallRoot(installRoot.getAbsolutePath());
            GlassFishRuntime gfr = GlassFishRuntime.bootstrap(bs);
            GlassFishProperties gfp = new GlassFishProperties(bs.getProperties());
            gfp.setInstanceRoot(instanceRoot.getAbsolutePath());
            GlassFish gf = gfr.newGlassFish(gfp);
            gf.start();
            String app = gf.getDeployer().deploy(appURI);
            System.out.println("Deployed " + app);
            gf.getDeployer().undeploy(app);
            gf.stop();
            gf.dispose();
            gfr.shutdown();
        }
    }
    private static URLClassLoader createCL(File gfHome) throws MalformedURLException {
        File cwd = new File(System.getProperty("user.dir"));
        final URL[] urls = {new File(gfHome, "lib/embedded/glassfish-embedded-shell.jar").toURI().toURL(),
                cwd.toURI().toURL()}; // add "." in classpath so that we can load Launcher class
        return new URLClassLoader(urls, ClassLoader.getSystemClassLoader().getParent());
    }
}
